using System;
using System.Collections.Generic;
using System.Collections.Immutable;
using System.Linq;
using System.Threading.Tasks;
using Autofac;
using BasicUnitTest.Mocking;
using FitConnect;
using FitConnect.Encryption;
using FitConnect.Interfaces.Subscriber;
using FitConnect.Models;
using FitConnect.Services.Interfaces;
using FitConnect.Services.Models;
using FitConnect.Services.Models.v1.Submission;
using FluentAssertions;
using Microsoft.Extensions.Logging;
using Moq;
using Newtonsoft.Json;
using NUnit.Framework;

namespace BasicUnitTest;

[TestFixture]
public class AutoRejectTests {
    [OneTimeSetUp]
    public void OneTimeSetup() {
    }

    [SetUp]
    public void SetUp() {
        var container = MockDependencyContainer.Container;
    }

    [TearDown]
    public void TearDown() {
    }

    [OneTimeTearDown]
    public void OneTimeTearDown() {
    }

    [Test]
    public void CreateSubmissionTest() {
        var uuid = Guid.NewGuid().ToString();


        var builder = MockDependencyContainer.Builder();

        var result = new SubmissionDto() {
            SubmissionId = uuid,
            DestinationId = Guid.NewGuid().ToString(),
            CaseId = Guid.NewGuid().ToString()
        };

        var submissionService = new Mock<ISubmissionService>();
        submissionService.Setup(
                s => s.GetSubmissionAsync(It.IsAny<string>()))
            .Returns((string id) => Task.FromResult<SubmissionDto>(result));
        builder.RegisterInstance<ISubmissionService>(submissionService.Object);

        var subscriber = Client.GetSubscriber(builder.Build());
        var exception = Assert.Throws<AggregateException>(() => {
            var intermediate = subscriber.RequestSubmissionAsync(uuid).Result;
        });

        exception!.InnerExceptions.First(e => e.GetType() == typeof(SecurityEventException)).Message
            .Should().Contain("submit");
    }

    [Test]
    [Ignore("Overkill")]
    public void SuccessFullRequestTest() {
        var builder = MockDependencyContainer.Builder();
        builder.RegisterInstance(new Mock<ILogger>().Object);

        var uuid = Guid.NewGuid().ToString();

        var encryptor = new FitEncryption(
            MockDependencyContainer.Container.Resolve<IFitConnectSettings>().PublicKeyEncryption,
            MockDependencyContainer.Container.Resolve<ILogger>());

        var submission = new Submission() {
            CaseId = Guid.NewGuid().ToString(),
            Id = uuid,
            DestinationId = Guid.NewGuid().ToString(),
            ServiceType = new() {
                Description = "Dummy service",
                Identifier = "url:leika:043049873549875398",
                Name = "Name"
            },
            Data = @"{""name"":""John Doe""}",
        };

        submission.EncryptedMetadata =
            encryptor.Encrypt(Sender.CreateMetadata((Submission)submission));

        var submissionService = new Mock<ISubmissionService>();
        submissionService.Setup(s =>
                s.GetStatusForSubmissionAsync(It.IsAny<string>(), It.IsAny<string>(), true))
            .Returns(() => Task.FromResult(
                new List<string> {
                    JsonConvert.SerializeObject(new SecurityEventToken() {
                        SubmissionId = uuid,
                        Events = new Events {
                            SubmitSubmissionEvent = new SubmitSubmissionEvent()
                        }
                    })
                }
            )!);
        builder.RegisterInstance<ISubmissionService>(submissionService.Object);

        var container = builder.Build();


        var subscriber = Client.GetSubscriber(container);
        var subscriberWithSubmission =
            (subscriber as Subscriber)!.CheckSubmissionAsync(submission).Result;
        subscriberWithSubmission.Submission!.Id.Should().Be(uuid);
    }
}
